﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ShoeStore.Entities;
using ShoeStore.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;

namespace ShoeStore.Repositories.Implementations;

internal class FactoryRepository : IFactoryRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<FactoryRepository> _logger;
    public FactoryRepository(IConnectionString connectionString,
    ILogger<FactoryRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public void CreateFactory(Factory factory)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(factory));
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
INSERT INTO Factorys (FactoryName, Manufacturer)
VALUES (@FactoryName, @Manufacturer)";
            connection.Execute(queryInsert, factory);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }
    public void UpdateFactory(Factory factory)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(factory));
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
UPDATE Factorys
SET
FactoryName=@FactoryName,
Manufacturer=@Manufacturer
WHERE [Id]=@Id";
            connection.Execute(queryUpdate, factory);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeleteFactory(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
DELETE FROM Factorys
WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public Factory ReadFactoryById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
SELECT * FROM Factorys
WHERE Id=@id";
            var factory = connection.QueryFirst<Factory>(querySelect,
            new { id });
            _logger.LogDebug("Найденный объект: {json}",
            JsonConvert.SerializeObject(factory));
            return factory;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public IEnumerable<Factory> ReadFactorys()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new
            SqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM Factorys";
            var factorys = connection.Query<Factory>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(factorys));
            return factorys;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
